# Демо-программа для взаимодействия с датчиком тока.
# Алгоритм демо-программы: сервопривод поворачивается по шагу налево (45°) [в этот момент считываются показания с датчика тока]
# затем сервопривод поворачивается по скважности вправо (90°) [в этот момент считываются показания с датчика тока]
# затем сервопривод поворачивается на угол 45°влево [в этот момент считываются показания с датчика тока]
import sys
from ctypes import *
import time
from ctypes.util import find_library
import platform
import os

# Подключаем внешнюю библиотеку для работы с SDK
libName = ""
platform = platform.system()
if platform == "Windows":
    libName = "librisdk.dll"
if platform == "Linux":
    libName = "librisdk.so"

libPath = os.getenv("LIB_RISDK")
print(libPath)

lib = cdll.LoadLibrary(libPath + libName)

# Объявление аргументов вызываемых функций.
lib.RI_SDK_InitSDK.argtypes = [c_int, c_char_p]
lib.RI_SDK_CreateModelComponent.argtypes = [c_char_p, c_char_p, c_char_p, POINTER(c_int), c_char_p]
lib.RI_SDK_LinkPWMToController.argtypes = [c_int, c_int, c_uint8, c_char_p]
lib.RI_SDK_LinkServodriveToController.argtypes = [c_int, c_int, c_int, c_char_p]

lib.RI_SDK_sensor_VoltageSensor_Voltage.argtypes = [c_int, POINTER(c_float), c_char_p]
lib.RI_SDK_exec_ServoDrive_Turn.argtypes = [c_int, c_int, c_int, c_bool, c_char_p]
lib.RI_SDK_sensor_VoltageSensor_Power.argtypes = [c_int, POINTER(c_float), c_char_p]
lib.RI_SDK_LinkVoltageSensorToController.argtypes = [c_int, c_int, c_uint8, c_char_p]
lib.RI_SDK_sensor_VoltageSensor_Sense.argtypes = [c_int, POINTER(c_float), POINTER(c_float), POINTER(c_float), POINTER(c_float), c_char_p]
lib.RI_SDK_sensor_VoltageSensor_VoltageShunt.argtypes = [c_int, POINTER(c_float), c_char_p]
lib.RI_SDK_sensor_VoltageSensor_Power.argtypes = [c_int, POINTER(c_float), c_char_p]
lib.RI_SDK_sensor_VoltageSensor_Current.argtypes = [c_int, POINTER(c_float), c_char_p]

lib.RI_SDK_sigmod_PWM_ResetAll.argtypes = [c_int, c_char_p]
lib.RI_SDK_DestroyComponent.argtypes = [c_int, c_char_p]
lib.RI_SDK_DestroySDK.argtypes = [c_bool, c_char_p]

# Объявление переменных для
errTextC = create_string_buffer(1000)   # Перменная, в которую будет записываться текст ошибки
errCode = c_int() # Переменная, в которую будет записываться код ошибки
i2c = c_int() # Переменная, в которой будет храниться дескриптор на объект i2c адаптера
pwm = c_int() # Переменная, в которой будет храниться дескриптор на объект pwm модулятора
voltageSensor = c_int()

# Переменныу, в которых будут храниться дескрипторы сервоприводов
servo = c_int()
voltage = c_float()
voltageShunt = c_float()
power = c_float()
current = c_float()

# main() - функция-"точка входа" в программах на Python и многих других языках.
def main():

    # Инициализация RI SDK.
    errCode = lib.RI_SDK_InitSDK(2, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Инициализация i2c адаптера.
    errCode = lib.RI_SDK_CreateModelComponent("connector".encode(), "i2c_adapter".encode(), "ch341".encode(), i2c, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Создание компонента датчика тока, напряжения и мощности модели ina219
    errCode = lib.RI_SDK_CreateModelComponent("sensor".encode(), "voltage_sensor".encode(), "ina219".encode(), voltageSensor, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Связывание i2c с датчиком тока, напряжения и мощности
    errCode = lib.RI_SDK_LinkVoltageSensorToController(voltageSensor, i2c, 0x41, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Инициализация ШИМ.
    errCode = lib.RI_SDK_CreateModelComponent("connector".encode(), "pwm".encode(), "pca9685".encode(), pwm, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Связывание ШИМ с i2c адаптером.
    errCode = lib.RI_SDK_LinkPWMToController(pwm, i2c, c_uint8(0x40), errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Инициализация сервопривода.
    errCode = lib.RI_SDK_CreateModelComponent("executor".encode(), "servodrive".encode(), "mg90s".encode(), servo, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Связывание сервопривода с ШИМ.
    errCode = lib.RI_SDK_LinkServodriveToController(servo, pwm, 0, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    def measureCurrentSensor():
        # Цикл в котором мы считываем показатели с датчика тока во время поворота сервопривода
        for i in range(3):
            # Данная остановка на каждой итерации необходима для считывания показателей датчика в разных положениях сервопривода
            time.sleep(1)
            print('Waiting 1s...')

            # Получение напряжения с датчика тока
            errCode = lib.RI_SDK_sensor_VoltageSensor_Voltage(voltageSensor, voltage, errTextC)
            if errCode != 0:
                print(errCode, errTextC.raw.decode())
                sys.exit(2)
            print("Voltage V: ", voltage.value)

            # Получение мощности с датчика тока
            errCode = lib.RI_SDK_sensor_VoltageSensor_Power(voltageSensor, power, errTextC)
            if errCode != 0:
                print(errCode, errTextC.raw.decode())
                sys.exit(2)
            print("Power P: ", power.value)

            # Получение напряжения на шунте с датчика тока
            errCode = lib.RI_SDK_sensor_VoltageSensor_VoltageShunt(voltageSensor, voltageShunt, errTextC)
            if errCode != 0:
                print(errCode, errTextC.raw.decode())
                sys.exit(2)
            print("Voltage shunt: ", voltageShunt.value)

            # Получение силы тока с дачика тока
            errCode = lib.RI_SDK_sensor_VoltageSensor_Current(voltageSensor, current, errTextC)
            if errCode != 0:
                print(errCode, errTextC.raw.decode())
                sys.exit(2)
            print("Current strength A: ", current.value)


    # Моментальный поворот сервопривода к значению угла, соответствующему длине управляющего импульса в 1000 микросекунд
    errCode = lib.RI_SDK_exec_ServoDrive_TurnByPulse(servo, 1000, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    print('\nПервое измерение показаний\n')
    measureCurrentSensor()

    # Вращение сервопривода на 90 градусов по часовой стрелке
    errCode = lib.RI_SDK_exec_ServoDrive_Turn(servo, 90, 100, True, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)


    print('\nВторое измерение показаний\n')
    measureCurrentSensor()

    # Моментальный поворот сервопривода к значению угла, соответствующему 300 шагам
    errCode = lib.RI_SDK_exec_ServoDrive_TurnByDutyCycle(servo, 300, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    print('\nТретье измерение показаний\n')
    measureCurrentSensor()

    # Сброс сигнала со всех портов ШИМ.
    errCode = lib.RI_SDK_sigmod_PWM_ResetAll(pwm, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Удаление из памяти компонента сервопривода.
    errCode = lib.RI_SDK_DestroyComponent(servo, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Удаление из памяти компонента ШИМ.
    errCode = lib.RI_SDK_DestroyComponent(pwm, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)
    # Удаление из памяти компонента i2c.
    errCode = lib.RI_SDK_DestroyComponent(i2c, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)
    # Завершение работы с RI SDK.
    errCode = lib.RI_SDK_DestroySDK(c_bool(True), errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    print("Success")

main()

